package org.logstash.bivalues;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.math.BigDecimal;
import java.math.BigInteger;
import org.jruby.RubyBignum;
import org.jruby.ext.bigdecimal.RubyBigDecimal;
import org.junit.Test;
import org.logstash.TestBase;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class BiValueTest extends TestBase {

    @Test
    public void testBigDecimalBiValueFromRuby() {
        BigDecimal s = BigDecimal.valueOf(12345.678D);
        BigDecimalBiValue subject = new BigDecimalBiValue(new RubyBigDecimal(ruby, s));
        assertTrue(subject.hasRubyValue());
        assertFalse(subject.hasJavaValue());
        assertEquals(s, subject.javaValue());
    }

    @Test
    public void testBigDecimalBiValueFromJava() {
        RubyBigDecimal v = new RubyBigDecimal(ruby, new BigDecimal(12345.678D));
        BigDecimalBiValue subject = new BigDecimalBiValue(new BigDecimal(12345.678D));
        assertFalse(subject.hasRubyValue());
        assertTrue(subject.hasJavaValue());
        assertEquals(v, subject.rubyValue(ruby));
    }

    @Test
    public void testNullBiValueFromJava() {
        NullBiValue subject = NullBiValue.newNullBiValue();
        assertTrue(subject.hasRubyValue());
        assertTrue(subject.hasJavaValue());
        assertEquals(ruby.getNil(), subject.rubyValue(ruby));
    }

    @Test
    public void testBigIntegerBiValueFromRuby() {
        BigInteger s = BigInteger.valueOf(12345678L);
        BigIntegerBiValue subject = new BigIntegerBiValue(new RubyBignum(ruby, s));
        assertTrue(subject.hasRubyValue());
        assertFalse(subject.hasJavaValue());
        assertEquals(s, subject.javaValue());
    }

    @Test
    public void testBigIntegerBiValueFromJava() {
        RubyBignum v = new RubyBignum(ruby, BigInteger.valueOf(12345678L));
        BigIntegerBiValue subject = new BigIntegerBiValue(BigInteger.valueOf(12345678L));
        assertFalse(subject.hasRubyValue());
        assertTrue(subject.hasJavaValue());
        assertEquals(v, subject.rubyValue(ruby));
    }

    @Test
    public void testSerialization() throws Exception {
        RubyBignum v = RubyBignum.newBignum(ruby, "-9223372036854776000");
        BiValue original = BiValues.newBiValue(v);

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ObjectOutputStream oos = new ObjectOutputStream(baos);
        oos.writeObject(original);
        oos.close();

        ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
        ObjectInputStream ois = new ObjectInputStream(bais);
        BiValue copy = (BiValue) ois.readObject();
        assertEquals(original, copy);
        assertFalse(copy.hasRubyValue());
    }
}
